/**
 * Ajax request abstraction module.
 *
 * @module request
 * @example
 * let { get, post } = peepso.modules.request;
 *
 * // The first request will automatically be aborted since the second request has the same ID.
 * get( 'fooRequest', 'endPoint', { foo: 'bar' } )
 *     .then( data => console.log( data ) )
 *     .catch( error => console.error( error ) );
 *
 * get( 'fooRequest', 'endPoint', { foo: 'quux' } );
 *     .then( data => console.log( data ) )
 *     .catch( error => console.error( error ) );
 *
 * post( 'anotherRequest', 'endPoint', { foo: 'bar' } );
 *     .then( data => console.log( data ) )
 *     .catch( error => console.error( error ) );
 *
 * // Requests with falsy ID will not be aborted.
 * post( null, 'endPoint', { foo: 'bar' } )
 *     .then( data => console.log( data ) )
 *     .catch( error => console.error( error ) );
 *
 * post( null, 'endPoint', { foo: 'bar' } )
 *     .then( data => console.log( data ) )
 *     .catch( error => console.error( error ) );
 */

import peepso from 'peepso';
import Promise from 'promise/lib/es6-extensions';

/**
 * Ongoing ajax requests.
 *
 * @type {Object.<XMLHttpRequest>}
 * @private
 */
const requests = {};

/**
 * Perform a get request.
 *
 * @param {string|null} xhrId
 * @param {string} url
 * @param  {Object} [data]
 * @returns {Promise}
 */
export function get( xhrId = null, url, data ) {
	if ( xhrId && requests[ xhrId ] ) {
		requests[ xhrId ].abort();
	}

	return new Promise( ( resolve, reject ) => {
		let xhr = peepso.getJson( url, data ),
			json;

		if ( xhrId ) {
			requests[ xhrId ] = xhr;
		}

		xhr.done( resp => ( json = resp ) );
		xhr.always( () => {
			if ( xhrId ) {
				delete requests[ xhrId ];
			}

			if ( json ) {
				resolve( json );
			} else {
				reject();
			}
		} );
	} );
}

/**
 * Perform a post request.
 *
 * @param {string|null} xhrId
 * @param {string} url
 * @param  {Object} [data]
 * @returns {Promise}
 */
export function post( xhrId = null, url, data ) {
	if ( xhrId && requests[ xhrId ] ) {
		requests[ xhrId ].abort();
	}

	return new Promise( ( resolve, reject ) => {
		let xhr = peepso.postJson( url, data ).ret,
			json;

		if ( xhrId ) {
			requests[ xhrId ] = xhr;
		}

		xhr.done( resp => ( json = resp ) );
		xhr.always( () => {
			if ( xhrId ) {
				delete requests[ xhrId ];
			}

			if ( json ) {
				resolve( json );
			} else {
				reject();
			}
		} );
	} );
}
